
import edu.mtech.esof427.keyboard.*;
import java.io.*;

public class KeyboardDecoder
{
    // Check if a command line option matches a switch 
    private static boolean isSwitch(String strParam, String strSwitch)
    {
        String strParamLower    = strParam.toLowerCase();
        String strSwitchLower   = strSwitch.toLowerCase();

        if (strParamLower.indexOf(strSwitchLower) == 0)
            return true;
        strSwitchLower = "-" + strSwitchLower;
        if (strParamLower.indexOf(strSwitchLower) == 0)
            return true;
        strSwitchLower = "-" + strSwitchLower;
        if (strParamLower.indexOf(strSwitchLower) == 0)
            return true;

        return false;
    }
    
    public static void main(String [] args) throws IOException
    { 
        if (args.length < 2)
        {
            System.out.println("KeyboardDecoder ");
            System.out.println("  -keyboard     <keyboard definition filename>");
            System.out.println("  -sequence     <trace sequence filename>");
            System.out.println("  -circle       (circular keyboard)");
            System.out.println("  -inside       (drop taps not strictly inside key)");
            System.out.println("  -focalMode    <0=avg, 1=first, 2=last>");
            System.out.println("  -touchFormat  <0=time, 1=finger>");
            
            System.out.println();
            return;
        }
        
        boolean circle = false;
        boolean inside = false;
        String keyboardFilename = "";
        String sequenceFilename = "";
        int mode = 0;
        int format = 0;
        
        // Parse the command line options
        int argc = args.length;
        int i = 0; 
        while (i < argc)
        {
            String nextParam  = "";
            String param = args[i];
            
            if (i < argc - 1)
                nextParam = args[i + 1];   

            if (isSwitch(param, "circle"))
            {
                circle = true;
            }
            else if (isSwitch(param, "inside"))
            {
                inside = true;
            }
            else if (isSwitch(param, "keyboard"))
            {
                keyboardFilename = nextParam;
                i++;
            }
            else if (isSwitch(param, "sequence"))
            {
                sequenceFilename = nextParam;
                i++;
            }
            else if (isSwitch(param, "focalmode"))
            {
                mode = Integer.parseInt(nextParam);
                i++;
            }
            else if (isSwitch(param, "touchformat"))
            {
                format = Integer.parseInt(nextParam);
                i++;
            }
            else 
            {
                System.out.println("Unknown switch: " + param);
            }
            i++;
        } 
        
        if ((keyboardFilename.length() == 0) || (sequenceFilename.length() == 0))
        {
            System.out.println("ERROR: must specify both -keyboard and -sequence");
            return;
        }

        System.out.println("Loading trace sequence from: " + sequenceFilename);
        TraceSequence sequence = new TraceSequence(sequenceFilename, format);        
        System.out.println("Traces in sequence: " + sequence.size());
        
        String result = "";
        System.out.println("Loading keyboard from: " + keyboardFilename);
        if (circle)
        {
            CircleKeyboard circleKeyboard = new CircleKeyboard(keyboardFilename);
            result = circleKeyboard.getText(sequence, mode, inside);
        }
        else
        {
            RectangleKeyboard rectangleKeyboard = new RectangleKeyboard(keyboardFilename);
            result = rectangleKeyboard.getText(sequence, mode, inside);
        }
        System.out.println("RESULT: " + result);
    }
}
