package edu.mtech.esof427.keyboard.test;

import static org.junit.Assert.*;
import org.junit.Test;
import edu.mtech.esof427.keyboard.*;

public class CircleKeyTest
{
    private static final double EPSILON = 1e-8;

    @Test
    public void testGetCharacter()
    {
        CircleKey key = new CircleKey("q,16,274,13");    
        assertTrue(key.getCharacter().equals("q"));
    }

    @Test(expected = RuntimeException.class)
    public void testConstructorTooFew()
    {
        CircleKey key = new CircleKey("q,16,274");        
    }

    @Test(expected = RuntimeException.class)
    public void testConstructorTooMany()
    {
        CircleKey key = new CircleKey("q,16,274,13,1,2");        
    }

    @Test(expected = RuntimeException.class)
    public void testConstructorBlank()
    {
        CircleKey key = new CircleKey("");        
    }

    @Test
    public void testIsInside()
    {
        CircleKey key = new CircleKey("q,16,274,13");    
        assertTrue(key.isInside(new Point(16, 274)));
        assertTrue(key.isInside(new Point(6, 274)));
        assertTrue(key.isInside(new Point(28, 274)));
        assertTrue(key.isInside(new Point(16, 264)));
        assertTrue(key.isInside(new Point(16, 287)));
    }

    @Test
    public void testIsInsideOutsidePoints()
    {
        CircleKey key = new CircleKey("q,16,274,13");    
        assertFalse(key.isInside(new Point(0, 274)));
        assertFalse(key.isInside(new Point(46, 274)));
        assertFalse(key.isInside(new Point(16, 288)));
        assertFalse(key.isInside(new Point(16, 260)));
    }

    @Test
    public void testGetDistanceToCenter()
    {
        CircleKey key = new CircleKey("q,100,100,10");
        assertEquals(0.0, key.getDistanceToCenter(new Point(100, 100)), EPSILON);
        assertEquals(10.0, key.getDistanceToCenter(new Point(90, 100)), EPSILON);
        assertEquals(10.0, key.getDistanceToCenter(new Point(110, 100)), EPSILON);
        assertEquals(10.0, key.getDistanceToCenter(new Point(100, 90)), EPSILON);
        assertEquals(10.0, key.getDistanceToCenter(new Point(100, 110)), EPSILON);
    }
    
    @Test
    public void testGetDistanceToCenterRandom()
    {
        CircleKey key = new CircleKey("q,0,0,10");

        for (int i = 0; i < 10000; i++)
        {
            double deltaX = 100.0 - Math.random() * 200;
            double deltaY = 100.0 - Math.random() * 200;

            double distance = key.getDistanceToCenter(new Point(deltaX, deltaY));
            
            double calculatedDistance = Math.sqrt(deltaX * deltaX + deltaY * deltaY);
            assertEquals(calculatedDistance, distance, EPSILON);
        }
    }

}
