package edu.mtech.esof427.keyboard.test;

import static org.junit.Assert.*;

import org.junit.Test;

import edu.mtech.esof427.keyboard.ComputeTrace;
import edu.mtech.esof427.keyboard.ComputeTraceFirst;
import edu.mtech.esof427.keyboard.Point;
import edu.mtech.esof427.keyboard.ScreenOrientation;
import edu.mtech.esof427.keyboard.TouchFinger;
import edu.mtech.esof427.keyboard.TouchTime;
import edu.mtech.esof427.keyboard.TraceOrientation;

public class ComputeTraceFirstTest
{
    private final static double EPSILON = 1e-10;

    @Test
    public void testGetFocalPointVerticalLine()
    {
        TraceOrientation trace = new TraceOrientation(ScreenOrientation.Portrait);        

        for (int i = -100; i <= 100; i++)
        {
            TouchTime time = new TouchTime(new Point(5.0, i), i + 1000);
            trace.add(time);
        }
        
        ComputeTrace compute = new ComputeTraceFirst();        
        Point focal = compute.getFocalPoint(trace);
        
        assertEquals(5.0, focal.getX(), EPSILON);
        assertEquals(-100.0, focal.getY(), EPSILON);
    }

    @Test
    public void testGetFocalPointHorizontalLine()
    {
        TraceOrientation trace = new TraceOrientation(ScreenOrientation.Portrait);        

        for (int i = -100; i <= 100; i++)
        {
            TouchTime time = new TouchTime(new Point(i, 5.0), i + 1000);
            trace.add(time);
        }
        
        ComputeTrace compute = new ComputeTraceFirst();        
        Point focal = compute.getFocalPoint(trace);
        
        assertEquals(-100.0, focal.getX(), EPSILON);
        assertEquals(5.0, focal.getY(), EPSILON);
    }
    
    @Test
    public void testGetFocalPointDiagonalLine()
    {
        TraceOrientation trace = new TraceOrientation(ScreenOrientation.Portrait);        

        for (int i = -100; i <= 100; i++)
        {
            TouchTime time = new TouchTime(new Point(i, i), i + 1000);
            trace.add(time);
        }
        
        ComputeTrace compute = new ComputeTraceFirst();        
        Point focal = compute.getFocalPoint(trace);
        
        assertEquals(-100.0, focal.getX(), EPSILON);
        assertEquals(-100.0, focal.getY(), EPSILON);
    }
    
    @Test
    public void testGetFocalPointDiagonalLineFinger()
    {
        TraceOrientation trace = new TraceOrientation(ScreenOrientation.Portrait);        

        for (int i = -100; i <= 100; i++)
        {
            TouchFinger time = new TouchFinger(new Point(i, i), i + 1000, 0);
            trace.add(time);
        }
        
        // Add one that would win except for being wrong finger
        TouchFinger time = new TouchFinger(new Point(-200, -200), 0, 1);
        trace.add(time);
               
        ComputeTrace compute = new ComputeTraceFirst();        
        Point focal = compute.getFocalPoint(trace);
        
        assertEquals(-100.0, focal.getX(), EPSILON);
        assertEquals(-100.0, focal.getY(), EPSILON);
    }
    
    @Test
    public void testGetFocalPointRandom()
    {
        TraceOrientation trace = new TraceOrientation(ScreenOrientation.Portrait);        

        double firstX = 0.0;
        double firstY = 0.0;
        long minTime = Long.MAX_VALUE;
        
        for (int i = 0; i < 10000; i++)
        {
            long ms = (long) (Math.random() * 100000);
            double x = Math.random();
            double y = Math.random();
            
            TouchTime time = new TouchTime(new Point(x, y), ms);
            trace.add(time);
            
            if (ms < minTime)
            {
                minTime = ms;
                firstX = x;
                firstY = y;
            }
        }
        
        ComputeTrace compute = new ComputeTraceFirst();        
        Point focal = compute.getFocalPoint(trace);
        
        assertEquals(firstX, focal.getX(), EPSILON);
        assertEquals(firstY, focal.getY(), EPSILON);
    }
    
}
